﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using HeuristicLab.Services.Hive.DataAccess;
using HeuristicLab.Services.Hive.Statistics.Models;

namespace HeuristicLab.Services.Hive.Statistics.Controllers {
  public class HomeController : Controller {
    public ActionResult Index() {
      using (var db = new HiveDataContext(Settings.Default.HeuristicLab_Hive_LinqConnectionString)) {
        var onlineSlaves = (from slave in db.Resources.OfType<Slave>()
                            where slave.SlaveState == SlaveState.Calculating || slave.SlaveState == SlaveState.Idle
                            select slave)
                           .ToList();

        var result = new OverallStatus {
          OverallCurrentlyAvailableCores = onlineSlaves.Sum(s => s.Cores ?? 0),
          CurrentlyAvailableCores = onlineSlaves.Where(s => s.IsAllowedToCalculate).Sum(s => s.Cores ?? 0),
          CurrentlyUsedCores = onlineSlaves.Sum(s => s.Cores ?? 0) - onlineSlaves.Sum(s => s.FreeCores ?? 0),
          CurrentlyJobsWaiting = db.Tasks.Count(x => x.State == TaskState.Waiting),
          OverallCpuUtilization = onlineSlaves.Any()
                                  ? Math.Round(onlineSlaves.Average(s => s.CpuUtilization), 2)
                                  : 0.0,
          AvailableCpuUtilization = onlineSlaves.Any(x => x.IsAllowedToCalculate)
                                    ? Math.Round(onlineSlaves.Where(x => x.IsAllowedToCalculate).Average(s => s.CpuUtilization), 2)
                                    : 0.0
        };


        return View(result);
      }
    }

    public ActionResult About() {
      ViewBag.Message = "Your app description page.";

      return View();
    }

    public ActionResult Error() {
      throw new Exception();
    }
  }
}
